%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
%
% psSimResult=psConvertSimOutput(psSimOutput,psSimStruct,caOffset)
%
% Interpolates and renames data according to simulation.    
% Arguments:
% psSimOutput    -  data generated from simulation och psModel 
function psSimResult=psConvertSimOutput(psSimOutput,psSimStruct)

   
  nanIndx = find(~isnan(psSimOutput.time));
  theNames= fieldnames(psSimOutput);
  for ii = 1:length(theNames)
      psSimOutput.(theNames{ii}) = psSimOutput.(theNames{ii})(nanIndx,:);
      psSimResult.rawData.(theNames{ii}) = psSimOutput.(theNames{ii});
  end
  
  psSimResult.rawData.thetaCyl = psSimOutput.theta;        
  psSimResult.rawData.t = psSimResult.rawData.time;
  % Calculate common pressure by takig mean value of doublets
  %   prepare ...
  [cbRows,cbCols]= find(~isnan(psSimOutput.pcb));
  [geRows,geCols]= find(~isnan(psSimOutput.pge));
  doubletsMatrix = ~isnan(psSimOutput.pcb)+...
      ~isnan(psSimOutput.pge);
  psSimOutput.pcb(find(isnan(psSimOutput.pcb))) = 0;
  psSimOutput.pge(find(isnan(psSimOutput.pge))) = 0;
  doubletsMatrix(find(doubletsMatrix==0))=1;
  psSimResult.rawData.p= (psSimOutput.pcb+psSimOutput.pge)./doubletsMatrix;

  
  interpolVarNames =...
      {'t','theta','p','pEM','pIM','TEM','TIM',...
       'V'};   
 
  % Interpolate rawData 
  simdatanames = interpolVarNames;
  resampMethod = 'cubic';
  % Always start at a fraction of a degree that fits with the resampleFac
  % Use theta as sample base
  thetaDiff=2*pi/psSimStruct.psSimPar.resampleFac;
  thetaStart=psSimStruct.psSimPar.thetaStart-...
      mod(psSimStruct.psSimPar.thetaStart,thetaDiff)+thetaDiff; 
  thetaStop=psSimStruct.psSimPar.thetaStop-...
      mod(psSimStruct.psSimPar.thetaStop,thetaDiff); 
  samplePoints = (thetaStart:thetaDiff:thetaStop)';
  theta = psSimResult.rawData.theta;
  % Add estimated time offset from data to simulations.
  for ii = 1:length(simdatanames)
      if psSimStruct.psSimPar.giveWarnings
          disp(sprintf('Interpolating data from: %s',simdatanames{ii}))
      end
      rawData = psSimResult.rawData.(simdatanames{ii});
      resampData=interp1(theta(:,1),rawData,samplePoints,resampMethod);
      psSimResult.resampData.(simdatanames{ii}) = resampData;
  end
  
  
  % Find the two last cycles of all cylinders
  lastThetaInt = [psSimResult.rawData.theta(end,:)-8*pi; psSimResult.rawData.theta(end,:)];

  [dummy minInd] = min(abs(psSimResult.rawData.theta-repmat(lastThetaInt(1,:),size(psSimResult.rawData.theta,1),1)));
  
  lastTwoCycleInd = min(minInd):size(psSimResult.rawData.theta,1);
   
  psSimResult.lastTwoCycles.p = psSimResult.rawData.p(lastTwoCycleInd,:);
  psSimResult.lastTwoCycles.pEM = psSimResult.rawData.pEM(lastTwoCycleInd,:);
  psSimResult.lastTwoCycles.pIM = psSimResult.rawData.pIM(lastTwoCycleInd,:);
  psSimResult.lastTwoCycles.V = psSimResult.rawData.V(lastTwoCycleInd,:);
  psSimResult.lastTwoCycles.Tge = psSimResult.rawData.Tge(lastTwoCycleInd,:);
  psSimResult.lastTwoCycles.Tu = psSimResult.rawData.Tu(lastTwoCycleInd,:);
  psSimResult.lastTwoCycles.Tb = psSimResult.rawData.Tb(lastTwoCycleInd,:);
  psSimResult.lastTwoCycles.theta = psSimResult.rawData.theta(lastTwoCycleInd,:);
  psSimResult.lastTwoCycleInd = lastTwoCycleInd;
  
  psSimResult.lastTwoCycles.theta = psSimResult.lastTwoCycles.theta-repmat(ceil(min(psSimResult.lastTwoCycles.theta)/4/pi)*4*pi,size(psSimResult.lastTwoCycles.theta,1),1);

  
  %  figure;
  % plot(psSimResult.rawData.theta(lastTwoCycleInd,:),psSimResult.rawData.uEV(lastTwoCycleInd,:));

  
  mDotCycle = zeros(1,4);
  for ii = 1:4
    IVC_ind=find(diff((psSimResult.rawData.uIV(lastTwoCycleInd,ii) ~= 0))==-1);
    IVO_ind=find(diff((psSimResult.rawData.uIV(lastTwoCycleInd,ii) ~= 0))==1);
    if(IVO_ind(1) < IVC_ind(1))
        IVO_ind = IVO_ind(2);
        IVC_ind = IVC_ind(2);
    else
        IVO_ind = IVO_ind(1);
        IVC_ind = IVC_ind(2);
    end
    mDotCycle(1,ii) = psSimResult.rawData.mTotIV(lastTwoCycleInd(IVC_ind+1),ii)-psSimResult.rawData.mTotIV(lastTwoCycleInd(IVO_ind),ii);
  end

  psSimResult.mFreshCycle = mDotCycle;
  psSimResult.mDotTot = sum(mDotCycle*psSimStruct.psSimPar.RPS/2);
  psSimResult.psSimStruct = psSimStruct;