%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

function [Xgc]=psCalcXgc(phi_react,x_react,phi_mix_b,x_mix_b,phi_mix_u,x_mix_u)

global psSimStruct
AFs=psSimStruct.psThermProp.AFs;

% Both x_react and x_mix_b goes into the x_b part. We cannot, however,
% have  both at the same time 
if any(x_react & x_mix_b)
   warning('psCalcXgc:InputError', '%s', ...
           ['You may not have both reaction parts and '...
            'burned mixing parts in the same zone! Also, it might not ' ...
            'be a good idea to have reaction parts and unburned mixing ' ...
            'parts in the same zone either.']) 
end

x_uf = x_mix_u./(AFs./phi_mix_u+1);
x_ua = x_mix_u-x_uf;
x_bf = x_mix_b./(AFs./phi_mix_b+1)+x_react./(AFs./phi_react+1);
x_ba = x_mix_b+x_react-x_bf;
Xgc = [x_uf x_ua x_bf  x_ba];


