%    This file is part of the psPack simulation package from the
%    doctoral dissertation "A DAE Formulation for Multi-Zone
%    Application to CVCP Engines" by Per Öberg. 
%    The dissertation is available at 
%    http://urn.kb.se/resolve?urn=urn:nbn:se:liu:diva-18120
%
%    Copyright (C) 2013  Per Öberg <per.oberg@liu.se>
%
%    This program is free software; you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation; either version 2 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License along
%    with this program; if not, write to the Free Software Foundation, Inc.,
%    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Plot the result
% Note that this plot is not the same as 2.4 
% This is because chepp isn't available 
% so that tables are used instead. Also, the 
% Lines for burned and unburned mixture 
% enthalpies are taken directly from the 
% simulation data instead of by generation using
% psThermProp or Chepp
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Convert sim-data
NaNIdx = find(isnan(psSimOut.dQ),1,'first');
clear h_b cp_b cv_b R_b dRdp_b dRdT_b dudp_b dudX_b dRdX
clear h_u cp_u cv_u R_u dRdp_u dRdT_u dudp_u dudX_u dRdX
clear xB;
for kk = 1:(NaNIdx)
    [h_b(kk) cp_b(kk) cv_b(kk) R_b(kk) dRdp_b(kk) dRdT_b(kk) dudp_b(kk) dudX_b(kk,:) dRdX(kk,:)]=psThermProp(psSimOut.pcb(kk),psSimOut.Tb(kk),psCalcReactXgc(psSimStruct.psModel.Xc0),ones(1,9));
    [h_u(kk) cp_u(kk) cv_u(kk) R_u(kk) dRdp_u(kk) dRdT_u(kk) dudp_u(kk) dudX_u(kk,:) dRdX(kk,:)]=psThermProp(psSimOut.pcb(kk),psSimOut.Tu(kk),psSimStruct.psModel.Xc0,ones(1,9));
    xB(kk) = psVibe(psSimOut.theta(kk));
end
Tb = psSimOut.Tb(1:NaNIdx)';
Tu = psSimOut.Tu(1:NaNIdx)';
Tav = Tu.*(1-xB)+Tb.*(xB);
u = (h_b-R_b.*Tb).*xB+(h_u-R_u.*Tu).*(1-xB);
h = h_b.*xB+h_u.*(1-xB);
phi= psCalcPhi(psSimStruct.psModel.Xc0);
phi=phi(2);
hf = figure;
clf
set(hf,'color','white');
subplot(3,1,1:2)
 
hold on;
h1 = plot(Tu,h_u-R_u.*Tu,'b--','LineWidth',2);
h2 = plot(Tb,h_b-R_b.*Tb,'r','LineWidth',2);
       
%grid on;
box off;
axis([200 1000 -0.5e6 0.2e6])
set(get(h1,'Parent'),'FontSize',14)
title(['Internal energy, Isooctane, \phi = ' ...
    num2str(phi) ', p=100kPa'])
xlabel('Temperature [K]')
ylabel('Specific internal energy [J/kg]')
        
line(Tav(1)*ones(1,2),1e5*[-1.0 -3.8],'Color','k')
line(Tav(1)*[0.98 1.0],-1e5*[1.0 0.85]/0.85,'Color','k')
line(Tav(1)*[1.0 1.02 ],-1e5*[0.85 1.0 ]/0.85,'Color','k')
line(Tav(1)*[0.98 1.0],-2.6*1e5+1e5*(-1.2)*[0.85 1.0],'Color','k')
line(Tav(1)*[1.0 1.02 ],-2.6*1e5+1e5*(-1.2)*[1.0 0.85 ],'Color','k')
ht2 = text(293.15,-2.5e5,{'x_f \cdot q_{HV}'},'FontSize',14, ...
    'HorizontalAlignment','right','VerticalAlignment','middle');

Tav_resamp = 299:(max(Tav(1:(end-1)))-299)/7:max(Tav(1:(end-1)));
Tav = Tav(1:(end-1)) ;
u = u(1:(end-1));

               
maxTavIdx1 = find(abs(max(Tav)- Tav)<1e-9,1,'first');
maxTavIdx2 = find(abs(max(Tav)- Tav)<1e-9,1,'last');
minTavIdx1 = find(abs(min(Tav(1:maxTavIdx1))- Tav(1:maxTavIdx1)) ...
    < 1e-9,1,'last');
minTavIdx2 = maxTavIdx2-1+find(abs(min(Tav(maxTavIdx2:end))- Tav(maxTavIdx2:end))<1e-9,1,'first');
u_resamp1 = interp1(Tav(minTavIdx1:maxTavIdx1),u(minTavIdx1:maxTavIdx1),Tav_resamp);
u_resamp2 = interp1(Tav(maxTavIdx2:minTavIdx2),u(maxTavIdx2:minTavIdx2),Tav_resamp);
h8 = plot([Tav_resamp Tav_resamp],[u_resamp1 u_resamp2],'x','LineWidth',2,'MarkerSize',9);
    

legend([h1 h2 h8],'Unburned','Burned','Simulation')